<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC3/modules/maintain_displayobj/DisplayObject.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Authors: Daniel Robert <robert@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: DisplayObject.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * DisplayObject.php - contains the {@link DisplayObject} class
  *
  * @author $Author: morgabra $
  * @filesource
  * @package    htdocs
  */

// for table row CSS classes, 'EVEN' and 'ODD' are digits
// for easy switching (++$class % 2)
define('EVEN', 0);
define('ODD', 1);

/**
  * This class is responsible for taking an object
  * and displaying its properties.  Rather than
  * writing separate classes for 'edit' and 'show' pages for each object,
  * this class simply takes an object, and an optional 'varname'=>'Display Label'
  * mapping, and generates a table with alternating CSS classes in each row.
  * If no mapping is given, this class will include all set class variables
  *
  * @package    htdocs
  */
class DisplayObject
{
    /** Map of 'varname'=>'Display Label' */
    var $map;

    /** Internally represented object */
    var $obj;

    /** Any variables to ignore */
    var $ignore;

    // {{{ constructor

    /**
      * Given a variable-to-display-name mapping and an object,
      * the class can generate a table representing the object
      *
      * @param  Object  the object to represent
      * @param  array   assoc. array of 'varname'=>'Label'
      * @param  array   list of variable names to not include
      */
    function DisplayObject(&$obj, $map = array(), $ignore = array())
    {
        // set the internal object
        $this->setObject($obj);

        // set the internal display mapping
        $this->setMap($map);

        // specify which variables to not display
        $this->setIgnore($ignore);

    }

    // }}}

    // {{{ setObject()

    /**
      * Setter
      *
      * @param  object  object to display
      */
    function setObject(&$obj)
    {
        $this->obj = $obj;
    }

    // }}}
    // {{{ setMap()

    /**
      * Set the map or build a default mapping if none was specified
      *
      * @param  array   varname=>'Display Label' mapping
      */
    function setMap($map)
    {
        if (!$map) {

            $arr = array();

            // use built-in ->toArray() if it exists,
            if (method_exists($this->obj, 'toArray'))
                $arr = $this->obj->toArray();

            // otherwise, just cast it
            else {
                $arr = array($this->obj);
                $arr = $arr[0];
            }

            // strip out '_' and convert to spaces
            foreach ($arr as $var=>$val)
            {
                if (!is_array($val) && strcmp($var{0}, '_'))
                {
                    $label = str_replace('_', ' ', $var);
                    $map[$var] = ucfirst($label);
                }
            }
        }

        $this->map = $map;
    }

    // }}}
    // {{{ setIgnore()

    /**
      * Specify which variables should not be included in the display
      *
      * @param  array   list of string names of variables to exclude
      */
    function setIgnore($ignore)
    {
        $this->ignore = $ignore;
    }

    // }}}

    // {{{ render()

    /**
      * The function to returns the result as HTML
      *
      * @return string  HTML representation of object
      */
    function render($indent_level = 1, $output_debug = 0)
    {
        $table = html_table();
        $table->set_class('displayobject');

        // class is either 'even' or 'odd'
        $class = EVEN;

        foreach($this->map as $var=>$val)
        {
            // skip any ignored variables
            if (in_array($var, $this->ignore)) {
                continue;
            }

            $data = '';
            if (method_exists($this->obj, 'get'.$var))
                $data = $this->obj->{"get$var"}();
            else
                $data = $this->obj->$var;

            // only display this row if there's data
            if (!$data || !strcasecmp('n', $var))
                continue;

            // show the label
            $label = html_td('displayobject_title');
            $label->add("$val:");

            // show the value
            $info = html_td('displayobject_value');
            $info->add($data);

            $tr = html_tr($this->getClass($class));
            $tr->add($label, $info);

            $table->add($tr);
        
            // switch classes
            $class = ++$class % 2;
        }

        return $table->render($indent_level, $output_debug);
    }

    // }}}
    // {{{ getClass()

    /**
      * Convert the class integer to a string
      */
    function getClass($class)
    {
        return (($class == EVEN) ? 'even' : 'odd');
    }

    // }}}

}
